/*
  sid (c) 1999-2004 by Dirk Jagdmann <doj@cubic.org>

  This file is part of sid.

  sid is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  sid is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  License for more details.

  You should have received a copy of the GNU General Public License
  along with sid; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
  $Header: /code/cw/sid/psid.cc,v 1.10 2004/05/14 14:51:04 doj Exp $
*/

extern char *errstr;

#include <netinet/in.h>

#include <string.h>
#include "psid.h"
#include "mus.hh"

static void strip(char *s)
{
  int len=strlen(s);
  while(len && s[len-1]==' ')
    s[--len]=0;
}

bool isPSID(FILE *f)
{
  if(f==0)
    return false;

  if(fseek(f, 0, SEEK_SET) < 0)
    {
      errstr="isPSID(): could not fseek to set";
      return false;
    }

  char buf[4];
  if(fread(buf, 4, 1, f) != 1)
    {
      errstr="isPSID(): could not read signature";
      return false;
    }

  if(!( (buf[0]=='P' || buf[0]=='R') && buf[1]=='S' && buf[2]=='I' && buf[3]=='D') )
    {
      return false;
    }

  return true;
}

psid* loadPSID(FILE *f, byte *mem)
{
  if(f==0 || mem==0)
    return 0;

  if(!isPSID(f))
    return 0;

  if(fseek(f, 0, SEEK_SET) < 0)
    {
      errstr="loadPSID(): could not fseek to set";
      return 0;
    }

  // load
  psid *ps=new psid;

  if(fread(ps, 0x7c, 1, f) != 1)
    {
      errstr="loadPSID(): could not read PSID header";
      goto error;
    }

  ps->version=ntohs(ps->version);
  if(ps->version > 2)
    {
      errstr="loadPSID(): illegal psid file version";
      goto error;
    }

  ps->dataOffset=ntohs(ps->dataOffset);
  if(!((ps->dataOffset==0x76 && ps->version==1)
       || (ps->dataOffset==0x7c && ps->version==2)))
    {
      errstr="loadPSID(): dataOffset does not match to psid version";
      goto error;
    }

  ps->loadAddr=ntohs(ps->loadAddr);
  ps->initAddr=ntohs(ps->initAddr);
  ps->playAddr=ntohs(ps->playAddr);

  ps->songTotal=ntohs(ps->songTotal);
  if(ps->songTotal == 0)
    {
      errstr="loadPSID(): no songTotal included";
      goto error;
    }

  ps->songStart=ntohs(ps->songStart);
  if(ps->songStart > 0)
    ps->songStart--;
  ps->songPlay=ps->songStart;

  ps->speed=ntohl(ps->speed);

  ps->name[31]=0; strip(ps->name);
  ps->author[31]=0; strip(ps->author);
  ps->copyright[31]=0; strip(ps->copyright);

  if(ps->version == 2)
    {
      ps->flags=ntohs(ps->flags);

      ps->MUSplayer=ps->flags&1;
      ps->PlaySIDSamples=ps->flags&2;
      ps->pal=true; //ps->flags&4; // pal is default
      // set ntsc only, when pal not set
      if(ps->flags&8 && !(ps->flags&4))
	ps->pal=false;
      ps->mos6581=ps->flags&16;
      ps->mos8580=ps->flags&32;
    }
  else
    {
      ps->flags=ps->reserved=0;
      ps->MUSplayer=false;
      ps->PlaySIDSamples=false;
      ps->pal=true;
      ps->mos6581=true;
      ps->mos8580=false;
    }

  if(fseek(f, ps->dataOffset, SEEK_SET) < 0)
    {
      errstr="loadPSID(): could not seek";
      goto error;
    }

  if(ps->MUSplayer)
    {
      byte buffer[65536];
      size_t size=fread(buffer, 1, sizeof(buffer), f);
      if(installMUS(ps, mem, buffer, size) < 0)
	goto error;
    }
  else
    {
      if(ps->loadAddr == 0)
	{
	  byte buf[2];      
	  if(fread(buf, 2, 1, f)!=1)
	    {
	      errstr="loadPSID(): error while reading ps->loadAddr";
	      goto error;
	    }

	  ps->loadAddr=lohi(buf[0], buf[1]);
	}

      if(ps->initAddr == 0)
	ps->initAddr = ps->loadAddr;

      fread(mem+ps->loadAddr, 1, 65536-ps->loadAddr, f);
    }

  return ps;

 error:
  delete ps;
  return 0;
}
