/*
  sidinfo (c) 1999-2004 by Dirk Jagdmann <doj@cubic.org>

  sidinfo is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  sidinfo is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with sidinfo; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
  $Header: /code/cw/sid/sidinfo_main.cc,v 1.4 2004/05/14 14:51:04 doj Exp $
*/

#include <iostream>

#include <curses.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>

using namespace std;

#include "c64.h"
#include "sidinfo.h"
#include "trackinfo.hpp"

char *errstr=0;
static C64 *c64=0;

static void showversion()
{
  cout << "INFO: sidinfo " VERSION " (c) 1999-2002 by Dirk Jagdmann" << endl;
}

static void showhelp()
{
  showversion();
  cout << "CMDL: sidinfo [options]" << endl;
  cout << "      options: -d dev      use dev as output device" << endl;
  cout << "               -h          print help" << endl;
  cout << "               -n num      show num SIDs" << endl;
  cout << "               -v          print version" << endl;
}

static void cleanup()
{
  if(c64)
    delete c64, c64=0;

  resetty();
  endwin();
  if(errstr)
    cerr << errstr << endl;
}

int main(int argc, char **argv)
{
  char *device=0;
  int num=1;

  int c=0;
  while(c!=EOF)
    {
      c=getopt(argc, argv, "d:hn:v");
      switch(c)
	{
	case 'd': device=optarg; break;
	case 'h': showhelp(); exit(0);
	case 'n': num=strtol(optarg, NULL, 0); break;
	case 'v': showversion(); exit(0);
	}
    }

  atexit(cleanup);

  int sid_fh=-1;
  if(device)
    sid_fh=open(device, O_RDONLY);
  else
    {
      sid_fh=open("/dev/sid", O_RDONLY);
      if(sid_fh<0)
	sid_fh=open("/dev/misc/sid", O_RDONLY);
    }
  if(sid_fh<0)
    {
      cerr << "could not open sid device" << endl;
      return 1;
    }

  C64 *c64=new C64;

  // enable all SIDs
  for(c=1; c<num && c<CWSID_MAX_NUM; c++)
    c64->setBase(c, 0xd400+c*0x40);

  initscr();
  savetty();
  raw();
  noecho();
  curs_set(FALSE);

  int xoffset=0, yoffset=0;

  trackinfo_init(xoffset, yoffset, c64);
  sidinfo_init(xoffset, yoffset, c64);

  while(true)
    {
      // wait for next IRQ on Sidblaster or keypress
      fd_set rfds;
      FD_ZERO(&rfds);
      FD_SET(0, &rfds);
      FD_SET(sid_fh, &rfds);

      struct timeval tv;
      tv.tv_sec = 0;
      tv.tv_usec = 20000;

      if(select(sid_fh+1, &rfds, NULL, NULL, &tv) < 0)
	{
	  perror("error while select");
	  exit(1);
	}

      if(FD_ISSET(0, &rfds))
	{
	  char c=getchar();
	  if(c=='q' || c==27)
	    exit(0);
	  if(c>='1' && c<='9')
	    trackinfo_scrollspeed(c-'0');

	  continue;
	}

      // read current sid registers
      allSID allsid;
      lseek(sid_fh, 0, SEEK_SET);
      read(sid_fh, &allsid, sizeof(allSID));
      c64->set(allsid);
      c64->setGateBits(true);	// fake gate bits, which can not be recognized, because we simply dump the sid state into c64

      trackinfo_print(c64);
      sidinfo_print(c64);
      doupdate();
    }

  return 0;
}
